"""
Functions and routines associated with Enasis Network Remote Connect.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Any
from typing import Literal
from typing import Optional
from typing import TYPE_CHECKING
from typing import get_args

from httpx import Response

from pydantic import BaseModel

from ..utils import HTTPClient

if TYPE_CHECKING:
    from .params import YouTubeParams



RESULT_KINDS = Literal[
    'channel',
    'playlist',
    'video']

_RESULT_KINDS = [
    f'youtube#{x}' for x in
    get_args(RESULT_KINDS)]



RESULT_VALUE = {
    'channelId': 'channel',
    'channelTitle': 'channel_title',
    'description': 'about',
    'publishedAt': 'published',
    'thumbnails': 'thumbnail',
    'title': 'title'}

RESULT_BASIC = {
    'channelId': 'channel',
    'kind': 'kind',
    'playlistId': 'playlist',
    'videoId': 'video'}



VIDEO_VALUE = {
    'channelId': 'channel',
    'channelTitle': 'channel_title',
    'description': 'about',
    'publishedAt': 'published',
    'thumbnails': 'thumbnail',
    'title': 'title'}



class YouTubeResult(BaseModel, extra='ignore'):
    """
    Contains information returned from the upstream response.

    .. note::
       Fields are not completely documented for this model.

    :param data: Keyword arguments passed to Pydantic model.
        Parameter is picked up by autodoc, please ignore.
    """

    kind: RESULT_KINDS

    channel: Optional[str] = None
    playlist: Optional[str] = None
    video: Optional[str] = None

    title: str
    about: Optional[str] = None
    channel_title: Optional[str] = None
    thumbnail: str
    published: str


    def __init__(
        self,
        **data: Any,
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """


        basic = data.get('id', {})

        items = RESULT_BASIC.items()

        for old, new in items:

            value = basic.get(old)

            if value is None:
                continue

            data[new] = value


        match = data.get('snippet', {})

        items = RESULT_VALUE.items()

        for old, new in items:

            value = match.get(old)

            if value is None:
                continue  # NOCVR

            data[new] = value


        if 'thumbnail' in data:
            data['thumbnail'] = (
                data['thumbnail']
                ['high']['url'])


        data['kind'] = data['kind'][8:]


        super().__init__(**data)



class YouTubeVideo(BaseModel, extra='ignore'):
    """
    Contains information returned from the upstream response.

    .. note::
       Fields are not completely documented for this model.

    :param data: Keyword arguments passed to Pydantic model.
        Parameter is picked up by autodoc, please ignore.
    """

    kind: RESULT_KINDS

    channel: str
    video: str

    title: str
    about: str
    thumbnail: str
    published: str


    def __init__(
        self,
        **data: Any,
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """


        data['video'] = data['id']


        video = data['snippet']

        items = VIDEO_VALUE.items()

        for old, new in items:

            value = video.get(old)

            if value is None:
                continue  # NOCVR

            data[new] = value


        if 'thumbnail' in data:
            data['thumbnail'] = (
                data['thumbnail']
                ['high']['url'])


        data['kind'] = data['kind'][8:]


        super().__init__(**data)



class YouTube:
    """
    Interact with the cloud service API with various methods.

    :param params: Parameters for instantiating the instance.
    """

    __params: 'YouTubeParams'
    __client: HTTPClient


    def __init__(
        self,
        params: 'YouTubeParams',
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """

        self.__params = params

        client = HTTPClient(
            timeout=params.timeout,
            verify=params.ssl_verify,
            capem=params.ssl_capem)

        self.__client = client


    @property
    def params(
        self,
    ) -> 'YouTubeParams':
        """
        Return the Pydantic model containing the configuration.

        :returns: Pydantic model containing the configuration.
        """

        return self.__params


    @property
    def client(
        self,
    ) -> HTTPClient:
        """
        Return the value for the attribute from class instance.

        :returns: Value for the attribute from class instance.
        """

        return self.__client


    def request_block(
        self,
        method: Literal['get'],
        path: str,
        params: Optional[dict[str, Any]] = None,
    ) -> Response:
        """
        Return the response for upstream request to the server.

        :param method: Method for operation with the API server.
        :param path: Path for the location to upstream endpoint.
        :param params: Optional parameters included in request.
        :returns: Response for upstream request to the server.
        """

        params = dict(params or {})

        server = 'www.googleapis.com'
        client = self.client

        params['key'] = (
            self.params.token)

        location = (
            f'https://{server}/'
            f'youtube/v3/{path}')

        request = client.request_block

        return request(
            method=method,
            location=location,
            params=params)


    async def request_async(
        self,
        method: Literal['get'],
        path: str,
        params: Optional[dict[str, Any]] = None,
    ) -> Response:
        """
        Return the response for upstream request to the server.

        :param method: Method for operation with the API server.
        :param path: Path for the location to upstream endpoint.
        :param params: Optional parameters included in request.
        :returns: Response for upstream request to the server.
        """

        params = dict(params or {})

        server = 'www.googleapis.com'
        client = self.client

        params['key'] = (
            self.params.token)

        location = (
            f'https://{server}/'
            f'youtube/v3/{path}')

        request = client.request_async

        return await request(
            method=method,
            location=location,
            params=params)


    def search(
        # NOCVR
        self,
        params: Optional[dict[str, Any]] = None,
    ) -> list[YouTubeResult]:
        """
        Return the results from the provided search parameters.

        :param params: Optional parameters included in request.
        :returns: Results from the provided search parameters.
        """

        return self.search_block(params)


    def search_block(
        self,
        params: Optional[dict[str, Any]] = None,
    ) -> list[YouTubeResult]:
        """
        Return the results from the provided search parameters.

        :param params: Optional parameters included in request.
        :returns: Results from the provided search parameters.
        """

        params = dict(params or {})


        if 'maxResults' not in params:
            params['maxResults'] = 50

        if 'order' not in params:
            params['order'] = 'date'

        if 'part' not in params:
            params['part'] = 'snippet,id'


        request = self.request_block

        response = request(
            'get', 'search', params)

        response.raise_for_status()

        fetched = response.json()

        assert isinstance(fetched, dict)


        return [
            YouTubeResult(**x)
            for x in fetched['items']
            if x['id']['kind']
            in _RESULT_KINDS]


    async def search_async(
        self,
        params: Optional[dict[str, Any]] = None,
    ) -> list[YouTubeResult]:
        """
        Return the results from the provided search parameters.

        :param params: Optional parameters included in request.
        :returns: Results from the provided search parameters.
        """

        params = dict(params or {})


        if 'maxResults' not in params:
            params['maxResults'] = 50

        if 'order' not in params:
            params['order'] = 'date'

        if 'part' not in params:
            params['part'] = 'snippet,id'


        request = self.request_async

        response = await request(
            'get', 'search', params)

        response.raise_for_status()

        fetched = response.json()

        assert isinstance(fetched, dict)


        return [
            YouTubeResult(**x)
            for x in fetched['items']
            if x['id']['kind']
            in _RESULT_KINDS]


    def videos(
        # NOCVR
        self,
        params: Optional[dict[str, Any]] = None,
    ) -> list[YouTubeVideo]:
        """
        Return the videos from the provided search parameters.

        :param params: Optional parameters included in request.
        :returns: Results from the provided search parameters.
        """

        return self.videos_block(params)


    def videos_block(
        self,
        params: Optional[dict[str, Any]] = None,
    ) -> list[YouTubeVideo]:
        """
        Return the videos from the provided search parameters.

        :param params: Optional parameters included in request.
        :returns: Results from the provided search parameters.
        """

        params = dict(params or {})


        if 'maxResults' not in params:
            params['maxResults'] = 50

        if 'order' not in params:
            params['order'] = 'date'

        if 'part' not in params:
            params['part'] = 'snippet,id'


        request = self.request_block

        response = request(
            'get', 'videos', params)

        response.raise_for_status()

        fetched = response.json()

        assert isinstance(fetched, dict)


        return [
            YouTubeVideo(**x)
            for x in fetched['items']]


    async def videos_async(
        self,
        params: Optional[dict[str, Any]] = None,
    ) -> list[YouTubeVideo]:
        """
        Return the videos from the provided search parameters.

        :param params: Optional parameters included in request.
        :returns: Results from the provided search parameters.
        """

        params = dict(params or {})


        if 'maxResults' not in params:
            params['maxResults'] = 50

        if 'order' not in params:
            params['order'] = 'date'

        if 'part' not in params:
            params['part'] = 'snippet,id'


        request = self.request_async

        response = await request(
            'get', 'videos', params)

        response.raise_for_status()

        fetched = response.json()

        assert isinstance(fetched, dict)


        return [
            YouTubeVideo(**x)
            for x in fetched['items']]


    def video_block(
        self,
        unique: str,
    ) -> YouTubeVideo:
        """
        Return the specific content within the social platform.

        :param unique: Unique identifier within social platform.
        :returns: Specific content within the social platform.
        """

        videos = self.videos_block(
            {'id': unique})

        return videos[0]


    async def video_async(
        self,
        unique: str,
    ) -> YouTubeVideo:
        """
        Return the specific content within the social platform.

        :param unique: Unique identifier within social platform.
        :returns: Specific content within the social platform.
        """

        videos = await (
            self.videos_async({
                'id': unique}))

        return videos[0]



"""
Functions and routines associated with Enasis Network Remote Connect.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from encommon import ENPYRWS
from encommon.types import inrepr
from encommon.types import instr
from encommon.utils import load_sample
from encommon.utils import prep_sample
from encommon.utils import read_text

from requests_mock import Mocker

from . import SAMPLES
from ..bridge import Bridge
from ..params import BridgeParams



def test_Bridge() -> None:
    """
    Perform various tests associated with relevant routines.
    """

    params = BridgeParams(
        server='192.168.1.10',
        token='mocked')

    bridge = Bridge(params)


    attrs = list(bridge.__dict__)

    assert attrs == [
        '_Bridge__params']


    assert inrepr(
        'bridge.Bridge object',
        bridge)

    assert hash(bridge) > 0

    assert instr(
        'bridge.Bridge object',
        bridge)


    assert bridge.params is params


    def _mocker_resource() -> None:

        server = params.server

        location = (
            f'https://{server}'
            '/clip/v2/resource')

        source = read_text(
            f'{SAMPLES}/source.json')

        mocker.get(
            url=location,
            text=source)


    with Mocker() as mocker:

        _mocker_resource()

        request = bridge.request

        response = request(
            'get', 'resource')


    response.raise_for_status()

    fetched = response.json()


    sample_path = (
        f'{SAMPLES}/dumped.json')

    sample = load_sample(
        sample_path, fetched,
        update=ENPYRWS)

    expect = prep_sample(fetched)

    assert sample == expect



"""
Functions and routines associated with Enasis Network Remote Connect.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from json import dumps
from json import loads
from unittest.mock import AsyncMock
from unittest.mock import patch

from encommon import ENPYRWS
from encommon.types import inrepr
from encommon.types import instr
from encommon.types.strings import SEMPTY
from encommon.utils import load_sample
from encommon.utils import prep_sample
from encommon.utils import read_text

from httpx import Request
from httpx import Response

from pytest import fixture
from pytest import mark

from . import SAMPLES
from ..params import RedditParams
from ..reddit import Reddit



_REQGET = Request('get', SEMPTY)
_REQPOST = Request('post', SEMPTY)



@fixture
def social() -> Reddit:
    """
    Construct the instance for use in the downstream tests.

    :returns: Newly constructed instance of related class.
    """

    params = RedditParams(
        username='mocked',
        password='mocked',
        client='mocked',
        secret='mocked',
        useragent='mocked')

    return Reddit(params)



def test_Reddit(
    social: Reddit,
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param social: Class instance for connecting to service.
    """


    attrs = list(social.__dict__)

    assert attrs == [
        '_Reddit__params',
        '_Reddit__client',
        '_Reddit__token']


    assert inrepr(
        'reddit.Reddit object',
        social)

    assert hash(social) > 0

    assert instr(
        'reddit.Reddit object',
        social)


    assert social.params is not None

    assert social.token is None



def test_Reddit_latest_block(
    social: Reddit,
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param social: Class instance for connecting to service.
    """


    patched = patch(
        'httpx.Client.request')

    with patched as mocker:

        _latest = read_text(
            f'{SAMPLES}/source.json')

        _token = read_text(
            f'{SAMPLES}/token.json')

        mocker.side_effect = [
            Response(
                status_code=200,
                content=_token,
                request=_REQPOST),
            Response(
                status_code=401,
                content=_latest,
                request=_REQGET),
            Response(
                status_code=200,
                content=_token,
                request=_REQPOST),
            Response(
                status_code=200,
                content=_latest,
                request=_REQGET)]

        latest = (
            social.latest_block(
                'mocked'))


    sample_path = (
        f'{SAMPLES}/latest.json')

    sample = load_sample(
        sample_path,
        [x.model_dump()
         for x in latest],
        update=ENPYRWS)

    expect = prep_sample([
        x.model_dump()
        for x in latest])

    assert sample == expect


    social.request_token_block()



@mark.asyncio
async def test_Reddit_latest_async(
    social: Reddit,
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param social: Class instance for connecting to service.
    """


    patched = patch(
        'httpx.AsyncClient.request',
        new_callable=AsyncMock)

    with patched as mocker:

        _latest = read_text(
            f'{SAMPLES}/source.json')

        _token = read_text(
            f'{SAMPLES}/token.json')

        mocker.side_effect = [
            Response(
                status_code=200,
                content=_token,
                request=_REQPOST),
            Response(
                status_code=401,
                content=_latest,
                request=_REQGET),
            Response(
                status_code=200,
                content=_token,
                request=_REQPOST),
            Response(
                status_code=200,
                content=_latest,
                request=_REQGET)]

        latest = await (
            social.latest_async(
                'mocked'))


    sample_path = (
        f'{SAMPLES}/latest.json')

    sample = load_sample(
        sample_path,
        [x.model_dump()
         for x in latest],
        update=ENPYRWS)

    expect = prep_sample([
        x.model_dump()
        for x in latest])

    assert sample == expect


    await social.request_token_async()



def test_Reddit_listing_block(
    social: Reddit,
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param social: Class instance for connecting to service.
    """


    patched = patch(
        'httpx.Client.request')

    with patched as mocker:

        _latest = read_text(
            f'{SAMPLES}/source.json')

        _token = read_text(
            f'{SAMPLES}/token.json')

        _listing = dumps([
            loads(_latest)])

        mocker.side_effect = [
            Response(
                status_code=200,
                content=_token,
                request=_REQPOST),
            Response(
                status_code=401,
                content=_listing,
                request=_REQGET),
            Response(
                status_code=200,
                content=_token,
                request=_REQPOST),
            Response(
                status_code=200,
                content=_listing,
                request=_REQGET)]

        listing = (
            social.listing_block(
                'mocked'))


    sample_path = (
        f'{SAMPLES}/listing.json')

    sample = load_sample(
        sample_path,
        listing.model_dump(),
        update=ENPYRWS)

    expect = prep_sample(
        listing.model_dump())

    assert sample == expect



@mark.asyncio
async def test_Reddit_listing_async(
    social: Reddit,
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param social: Class instance for connecting to service.
    """


    patched = patch(
        'httpx.AsyncClient.request',
        new_callable=AsyncMock)

    with patched as mocker:

        _latest = read_text(
            f'{SAMPLES}/source.json')

        _token = read_text(
            f'{SAMPLES}/token.json')

        _listing = dumps([
            loads(_latest)])

        mocker.side_effect = [
            Response(
                status_code=200,
                content=_token,
                request=_REQPOST),
            Response(
                status_code=401,
                content=_listing,
                request=_REQGET),
            Response(
                status_code=200,
                content=_token,
                request=_REQPOST),
            Response(
                status_code=200,
                content=_listing,
                request=_REQGET)]

        listing = await (
            social.listing_async(
                'mocked'))


    sample_path = (
        f'{SAMPLES}/listing.json')

    sample = load_sample(
        sample_path,
        listing.model_dump(),
        update=ENPYRWS)

    expect = prep_sample(
        listing.model_dump())

    assert sample == expect



"""
Functions and routines associated with Enasis Network Remote Connect.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from unittest.mock import AsyncMock
from unittest.mock import patch

from encommon.types import inrepr
from encommon.types import instr

from httpx import Response

from pytest import fixture
from pytest import mark

from ..http import HTTPClient



@fixture
def client() -> HTTPClient:
    """
    Construct the instance for use in the downstream tests.

    :returns: Newly constructed instance of related class.
    """

    return HTTPClient()



def test_HTTPClient(
    client: HTTPClient,
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param client: Class instance for connecting with server.
    """


    attrs = list(client.__dict__)

    assert attrs == [
        '_HTTPClient__timeout',
        '_HTTPClient__headers',
        '_HTTPClient__verify',
        '_HTTPClient__capem',
        '_HTTPClient__httpauth',
        '_HTTPClient__retry',
        '_HTTPClient__backoff',
        '_HTTPClient__states',
        '_HTTPClient__client_block',
        '_HTTPClient__client_async']


    assert inrepr(
        'http.HTTPClient object',
        client)

    assert hash(client) > 0

    assert instr(
        'http.HTTPClient object',
        client)


    assert client.timeout == 30

    assert client.headers is None

    assert client.verify is True

    assert client.capem is None

    assert client.httpauth is None

    assert client.retry == 3

    assert client.backoff == 3.0

    assert client.states == {429}

    assert client.client_block is not None

    assert client.client_async is not None



def test_HTTPClient_block(
    client: HTTPClient,
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param client: Class instance for connecting with server.
    """

    patched = patch(
        'httpx.Client.request')

    request = client.request_block

    with patched as mocker:

        mocker.side_effect = [
            Response(429),
            Response(200)]

        response = request(
            'get', 'https://enasis.net')

        status = response.status_code

        assert status == 200

        assert mocker.call_count == 2



@mark.asyncio
async def test_HTTPClient_async(
    client: HTTPClient,
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param client: Class instance for connecting with server.
    """

    patched = patch(
        'httpx.AsyncClient.request',
        new_callable=AsyncMock)

    request = client.request_async

    with patched as mocker:

        mocker.side_effect = [
            Response(429),
            Response(200)]

        response = await request(
            'get', 'https://enasis.net')

        status = response.status_code

        assert status == 200

        assert mocker.call_count == 2



"""
Functions and routines associated with Enasis Network Remote Connect.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from encommon import ENPYRWS
from encommon.types import inrepr
from encommon.types import instr
from encommon.utils import load_sample
from encommon.utils import prep_sample
from encommon.utils import read_text

from requests_mock import Mocker

from . import SAMPLES
from ..params import RouterParams
from ..router import Router



def test_Router() -> None:
    """
    Perform various tests associated with relevant routines.
    """

    params = RouterParams(
        server='192.168.1.1',
        username='mocked',
        password='mocked')

    router = Router(params)


    attrs = list(router.__dict__)

    assert attrs == [
        '_Router__params',
        '_Router__session']


    assert inrepr(
        'router.Router object',
        router)

    assert hash(router) > 0

    assert instr(
        'router.Router object',
        router)


    assert router.params is params

    assert router.session is not None


    def _mocker_cookie() -> None:

        server = params.server

        location = (
            f'https://{server}'
            '/api/auth/login')

        mocker.post(location)


    def _mocker_users() -> None:

        server = params.server

        location = (
            f'https://{server}'
            '/proxy/network'
            '/api/s/default/rest/user')

        source = read_text(
            f'{SAMPLES}/source.json')

        mocker.register_uri(
            'get', location,
            [{'text': source,
              'status_code': 401},
             {'text': source}])


    with Mocker() as mocker:

        _mocker_cookie()
        _mocker_users()

        request = router.request_proxy

        response = request(
            'get', 'rest/user')


    response.raise_for_status()

    fetched = response.json()


    sample_path = (
        f'{SAMPLES}/dumped.json')

    sample = load_sample(
        sample_path, fetched,
        update=ENPYRWS)

    expect = prep_sample(fetched)

    assert sample == expect



"""
Functions and routines associated with Enasis Network Remote Connect.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Any
from typing import Optional
from typing import TYPE_CHECKING

from requests import Response
from requests import request

if TYPE_CHECKING:
    from .params import BridgeParams



class Bridge:
    """
    Interact with the cloud service API with various methods.

    :param params: Parameters for instantiating the instance.
    """

    __params: 'BridgeParams'


    def __init__(
        self,
        params: 'BridgeParams',
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """

        self.__params = params


    @property
    def params(
        self,
    ) -> 'BridgeParams':
        """
        Return the Pydantic model containing the configuration.

        :returns: Pydantic model containing the configuration.
        """

        return self.__params


    def request(
        self,
        method: str,
        path: str,
        params: Optional[dict[str, Any]] = None,
        json: Optional[dict[str, Any]] = None,
    ) -> Response:
        """
        Return the response for upstream request to the server.

        :param method: Method for operation with the API server.
        :param path: Path for the location to upstream endpoint.
        :param params: Optional parameters included in request.
        :param json: Optional JSON payload included in request.
        :returns: Response for upstream request to the server.
        """

        params = dict(params or {})
        json = dict(json or {})

        server = self.params.server
        token = self.params.token
        verify = self.params.ssl_verify
        capem = self.params.ssl_capem

        token_key = 'hue-application-key'

        location = (
            f'https://{server}'
            f'/clip/v2/{path}')

        return request(
            method=method,
            url=location,
            timeout=self.params.timeout,
            params=params,
            json=json,
            headers={token_key: token},
            verify=capem or verify)
